
#include <OneWire.h>

OneWire  ds(P2_2); 

void setup(void) {
  Serial.begin(9600);
}

void loop(void) {
  byte i;
  byte present = 0;
  byte type_s;
  byte data[12];
  byte addr[8];
  float celsius, fahrenheit;
  
  if ( !ds.search(addr)) {
    Serial.println("No more addresses.");
    Serial.println();
    ds.reset_search();
    delay(10000);
    return;
  }
  
  Serial.print("ROM =");
  for( i = 0; i < 8; i++) {
    Serial.write(' ');
    Serial.print(addr[i], HEX);
  }

  if (OneWire::crc8(addr, 7) != addr[7]) {
      Serial.println("CRC is not valid!");
      delay(1000);
      return;
  }
  Serial.println();
 
  // the first ROM byte indicates which chip
  switch (addr[0]) {
    case 0x10:
      Serial.println("  Chip = DS18S20");  // or old DS1820
      type_s = 1;
      break;
    case 0x28:
      Serial.println("  Chip = DS18B20");
      type_s = 0;
      break;
    case 0x22:
      Serial.println("  Chip = DS1822");
      type_s = 0;
      break;
    default:
      Serial.println("Device is not a DS18x20 family device.");
      return;
  } 

  ds.reset(); //it's better to test ret code here... 
  ds.select(addr);
  ds.write(0x44,1);         // start conversion, with parasite power on at the end
  //ds.write(0x44,0);         // start conversion, with Vdd on at the end  
  delay(1000);     // depends on resolution, up to 750ms
  ds.depower();    // to stop parasite power
  
  ds.reset();
  ds.select(addr);    
  ds.write(0xBE);         // Read Scratchpad
  
  Serial.print("  Data = ");
  Serial.print(present,HEX);
  Serial.print(" ");
  for ( i = 0; i < 9; i++) {           // we need 9 bytes
    data[i] = ds.read();
    Serial.print(data[i], HEX);
    Serial.print(" ");
  }
  Serial.print(" CRC=");
  Serial.print(OneWire::crc8(data, 8), HEX);
  Serial.println();

  if (OneWire::crc8(data, 8) != data[8]) { 
    Serial.println("CRC is not valid!");
    delay(100);
    return;
  }

  // convert the data to actual temperature
  int bits=0;
  unsigned int raw = (data[1] << 8) | data[0];
  
  if (type_s) {
    // raw is a number of 0.5 C counts
    raw = raw << 3; // 9 bit resolution default
    bits=9;
    if (data[7] == 0x10) {
      // count remain gives full 12 bit resolution
      raw = (raw & 0xFFF0) + 12 - data[6]; 
      bits=12;
    }
  } else {    
    byte cfg = (data[4] & 0x60);
    bits=12;
    if (cfg == 0x00) { bits=9; }// 9 bit resolution, 93.75 ms
    else if (cfg == 0x20) { bits=10; }// 10 bit res, 187.5 ms
    else if (cfg == 0x40) { bits = 11; }// 11 bit res, 375 ms
    // default is 12 bit resolution, 750 ms conversion time
  }
  
  int16_t r = raw;
  
  celsius = (float)r / 16.0;
  fahrenheit = celsius * 1.8 + 32.0;
  Serial.print("  Bits = ");
  Serial.print(bits);
  Serial.print("  Temperature = ");
  Serial.print(celsius);
  Serial.print(" Celsius, ");
  Serial.print(fahrenheit);
  Serial.println(" Fahrenheit");
}
